<?php
/**
 * @file
 * API documenations for Party Open CRM.
 */

/**
 * Define namespaces for the Facet API Session URL processor.
 *
 * @return array
 *   An array of paths suitable for drupal_match_path() keyed by the namespace
 *   identifier, outer keyed by the searcher they apply to. Global namespaces
 *   can be declared using an outer key of NULL. The namespace identified is
 *   comprised of alphanumeric characters and underscores. It can end with |INT
 *   where INT is an integer value indicating the namespace is sensitive the
 *   first INT parts of the path.
 *
 * @see hook_opencrm_facet_namespace_patterns_alter()
 */
function hook_opencrm_facet_namespace_patterns() {
  return array(
    'search_api@default_node_index' => array(
      'content_browser' => "node",
      'content_admin' => "admin/content\nadmin/content/*",
    ),
  );
}

/**
 * Alter namespaces for the Facet API Session URL processor.
 *
 * @param array $namespaces
 *   An array of paths suitable for drupal_match_path() keyed by the namespace
 *   identifier, outer keyed by the searcher they apply to. Global namespaces
 *   use an outer key of NULL.
 *
 * @see hook_opencrm_facet_namespace_patterns()
 */
function hook_opencrm_facet_namespace_patterns_alter(&$namespaces) {
  // Make all admin facets remembered in the session.
  $namespaces[NULL]['admin'] = "admin\nadmin/*";
}

/**
 * Provide information for the party archive form.
 *
 * @param Party $party
 *   The party the archive form is for.
 * @param boolean $archiving
 *   TRUE if the party is being archived, FALSE if it's being unarchived.
 *
 * @return array
 *   An array of information items suitable for theme_item_list(). Keys are
 *   recommended to ease altering. Outer keyed by one of the following
 *   categories (in order of display):
 *     - info: Information that is helpful to know about archiving.
 *     - warnings: Warning that will be displayed in red.
 *     - actions: Actions that the user should take before archiving a party.
 *       If an item is passed as an array, the following keys can be set:
 *         - #required: Indicates the action is required.
 *         - #completed: Indicates the action has been completed.
 *       If an action is required but not completed the submit button will be
 *       disabled.
 *     - auto: Automated processes that will happen when a party is archived.
 *
 * @see hook_opencrm_archive_info_alter()
 */
function hook_opencrm_archive_info($party, $archiving) {
  if ($archiving) {
    $has_web_account = count($party->getDataSetController('user')->getEntityIds()) == 0;
    return array(
      'info' => array(
        'hide' => t('Archiving this party will hide it for users without the view archived parties permission.'),
      ),
      'warning' => array(
        'comms' => t('Communications with this party will be prevented.'),
      ),
      'actions' => array(
        'check' => t('Make sure the party should be archived.'),
        'account' => array(
          'data' => t('Block, delete or detach any connected web accounts.'),
          '#required' => TRUE,
          '#completed' => $has_web_account,
        ),
      ),
    );
  }
}

/**
 * Alter information for the party archive form.
 *
 * @param array $info
 *   An array of item list arrays to be outputted.
 * @param Party $party
 *   The party the archive form is for.
 * @param boolean $archiving
 *   TRUE if the party is being archived, FALSE if it's being unarchived.
 *
 * @see hook_opencrm_archive_info()
 */
function hook_opencrm_archive_info_alter(&$info, $party, $archiving) {
  if ($archiving) {
    $info['actions']['opencrm_account']['#required'] = FALSE;
  }
}
