<?php
/**
 * @file
 *  Custom Pages defined by the opencrm module.
 */

/**
 * Form for merging two parties.
 */
function opencrm_party_merge_form($form, &$form_state, $primary_party = FALSE, $secondary_party = FALSE) {
  $form['description'] = array(
    '#type' => 'markup',
    '#markup' => t('Use this form to merge two parties together. This will
      merge all the information on the Secondary party into the primary party.
      Where there are conflicts, information on the primary party will take
      precedent.'),
    '#weight' => -5,
  );

  $form['primary_party'] = array(
    '#type' => 'textfield',
    '#title' => t('Primary Party Id'),
    '#description' => t('The party id of the primary party (see above).'),
    '#required' => TRUE,
    '#default_value' => !empty($primary_party) ? $primary_party : NULL,
  );

  $form['secondary_party'] = array(
    '#type' => 'textfield',
    '#title' => t('Secondary Party Id'),
    '#description' => t('The party id of the secondary party (see above).'),
    '#required' => TRUE,
    '#default_value' => !empty($secondary_party) ? $secondary_party : NULL,
  );

  $form['actions'] = array(
    '#type' => 'actions',
    '#weight' => 100,
  );

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Merge'),
    '#validate' => array(
      'opencrm_party_merge_form_validate',
    ),
    '#submit' => array(
      'opencrm_party_merge_form_submit',
    ),
  );

  $form['status'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#title' => t('Status'),
    '#weight' => 150,
  );

  $count = db_select('entity_merge_semaphore', 'ems')
    ->isNull('finish_time')
    ->groupBy('entity_id_1')
    ->groupBy('entity_id_2')
    ->condition('entity_type', 'party')
    ->countQuery()
    ->execute()
    ->fetchField();

  if ($count) {
    $message = t('There are currently @count merges queued for processing. Use the button below to process a batch of merges.', array('@count' => $count));
  }
  else {
    $message = t('No merges currently queued.');
  }

  $form['status']['status'] = array(
    '#markup' => $message,
    '#prefix' => '<div class="form-item clearfix">',
    '#suffix' => '</div>',
  );

  $form['status']['process'] = array(
    '#type' => 'submit',
    '#limit_validation_errors' => array(),
    '#value' => t('Process Merges'),
    '#description' => t('Process a batch of merge processes.'),
    '#submit' => array(
      'opencrm_party_merge_form_process',
    ),
    '#access' => $count,
  );

  return $form;
}

/**
 * Form validation for merging two parties.
 */
function opencrm_party_merge_form_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['primary_party']) || !party_load($form_state['values']['primary_party'])) {
    form_error($form['primary_party'], t('Please enter a valid party id for the primary party.'));
  }

  if (!is_numeric($form_state['values']['secondary_party']) || !party_load($form_state['values']['secondary_party'])) {
    form_error($form['secondary_party'], t('Please enter a valid party id for the secondary party.'));
  }
}

/**
 * Form submission for merging two parties.
 */
function opencrm_party_merge_form_submit($form, &$form_state) {
  $primary = party_load($form_state['values']['primary_party']);
  $secondary = party_load($form_state['values']['secondary_party']);

  entity_merge($primary, $secondary, 'party');

  drupal_set_message(t('%secondary has been queued for merging with %primary.', array('%secondary' => $secondary->label, '%primary' => $primary->label)));

  $form_state['redirect'] = 'admin/party/' . $primary->pid;
}

/**
 * Form submission for processing a batch of merges.
 */
function opencrm_party_merge_form_process($form, &$form_state) {
  // Get the time at the start.
  $start_time = microtime(TRUE);

  // Get the queue
  $queue = DrupalQueue::get('entity_merge');
  $count = 0;

  while ((microtime(TRUE) - $start_time) < 29 && $item = $queue->claimItem()) {
    $proc = $item->data;
    $proc->prepare()->go();
    $queue->deleteItem($item);
    $count++;
  }

  drupal_set_message(t('Ran @count merge processes.', array('@count' => $count)));
}

/**
 * Page Callback for the Email preview page.
 */
function opencrm_email_preview($node) {
  // Only work on newsletter emails.
  if ($node->type != 'simplenews') {
    drupal_not_found();
  }

  if (!empty($node->panelizer['email_html']->display)) {
    $node->panelizer['email_html']->pipeline = 'ipe';
    $node->panelizer['email_html']->display->renderer = 'ipe';
  }

  return node_view($node, 'email_html');
}

/**
 * Page Callback to request rights to edit an organisation.
 */
function opencrm_request_edit_page($org, $indiv = NULL) {
  global $user;

  opencrm_request_editor_status($org, $indiv);
  drupal_goto('user/' . $user->uid . '/related-organisations');
}

/**
 * Page callback for marking a party for review.
 */
function opencrm_party_review($party, $review, $js, $token = NULL) {
  // Check the action makes sense.
  $wrapper = entity_metadata_wrapper('party', $party);
  if ($wrapper->party_needs_review->value() == $review) {
    return MENU_ACCESS_DENIED;
  }

  if ($js) {
    ctools_include('ajax');
    ctools_include('modal');
  }

  // If we have a token, validate and process immediately.
  if ($token) {
    if (!drupal_valid_token($token, 'party_review:' . $party->pid)) {
      return MENU_ACCESS_DENIED;
    }

    $wrapper->party_needs_review = $review;
    $wrapper->save();

    if ($js) {
      return array(
        '#type' => 'ajax',
        '#commands' => array(
          ctools_ajax_command_redirect('admin/party/' . $party->pid),
        ),
      );
    }
    else {
      drupal_goto('admin/party/' . $party->pid);
    }
  }

  // Otherwise output a confirm form.
  $form_state = array();
  $form_state['build_info']['args'] = array($wrapper);
  form_load_include($form_state, 'inc', 'opencrm', 'opencrm.pages');

  if ($js) {
    $form_state['ajax'] = TRUE;
    if ($review) {
      $form_state['title'] = t('Do you want to mark %party for review?', array(
        '%party' => $party->label,
      ));
    }
    else {
      $form_state['title'] = t('Do you want to un-mark %party for review?', array(
        '%party' => $party->label,
      ));
    }

    return array(
      '#type' => 'ajax',
      '#commands' => ctools_modal_form_wrapper('opencrm_party_review_form', $form_state),
    );
  }
  else {
    return drupal_build_form('opencrm_party_review_form', $form_state);
  }
}

/**
 * Form constructor for the party mark for review form.
 *
 * @param $party
 *   The wrapped party we are acting on.
 *
 * @see opencrm_party_review_form_submit().
 */
function opencrm_party_review_form($form, &$form_state, $party) {
  $form['#party'] = $party;
  $form['needs_review'] = array(
    '#type' => 'value',
    '#value' => $party->party_needs_review->value() ? 0: 1,
  );
  if ($form['needs_review']['#value']) {
    $question = t('Do you want to mark %party for review?', array('%party' => $party->label()));
    $description = t('This will show a message on the party dashboard and request the user to update their details next time they log in.');
  }
  else {
    $question = t('Do you want to un-mark %party for review?', array('%party' => $party->label()));
    $description = '';
  }
  return confirm_form($form, $question, 'admin/party/' . $party->getIdentifier(), $description);
}

/**
 * Submission handler for opencrm_party_review_form().
 */
function opencrm_party_review_form_submit(&$form, &$form_state) {
  $form['#party']->party_needs_review = $form_state['values']['needs_review'];
  $form['#party']->save();
  $form_state['redirect'] = 'admin/party/' . $form['#party']->getIdentifier();
}

/**
 * Form constructor for the leave organisation form.
 *
 * @param Party $party
 *   The group party we want to leave membership of.
 * @param $account
 *   Optionally an user who's membership we want to end. If not given, the
 *   current user will be used.
 *
 * @see opencrm_party_organisation_leave_form_submit().
 */
function opencrm_party_organisation_leave_form($form, &$form_state, $party, $account = NULL) {
  if (!$account) {
    global $user;
    $account = $user;
  }

  $form['#group'] = $party;
  $form['#account'] = $account;
  $form['#member'] = party_user_get_party($account);

  $question = t('Are you sure you want to leave %group?', array('%group' => $party->label));
  $path = 'user/' . $account->uid . '/related-organisations';
  return confirm_form($form, $question, $path, '', t('Leave'), t('Cancel'));
}

/**
 * Submission handler for opencrm_party_organisation_leave_form().
 */
function opencrm_party_organisation_leave_form_submit($form, &$form_state) {
  og_ungroup('party', $form['#group']->pid, 'party', $form['#member']->pid);
  $form_state['redirect'] = 'user/' . $form['#account']->uid . '/related-organisations';
}
