<?php

/**
 * @file
 * Plugin to provide an relationship handler that uses relation entity instances.
 */

/**
 * Plugins are described by creating a $plugin array which will be used
 * by the system that includes this file.
 */
$plugin = array(
  'title' => t('Relation: entity from entity'),
  'description' => t('Creates an entity context from a relation entity instance endpoint.'),
  'context' => 'relation_ctools_relationships_entity_from_entity_context',
  'get child' => 'relation_ctools_relationships_entity_from_entity_get_child',
  'get children' => 'relation_ctools_relationships_entity_from_entity_get_children',
  'defaults' => array('delta' => 0),
);

function relation_ctools_relationships_entity_from_entity_get_child($plugin, $parent, $child) {
  $plugins = relation_ctools_relationships_entity_from_entity_get_children($plugin, $parent);
  return $plugins[$parent . ':' . $child];
}

function relation_ctools_relationships_entity_from_entity_get_children($parent_plugin, $parent) {
  $cid = $parent_plugin['name'] . ':' . $parent;
  $cache = &drupal_static(__FUNCTION__);
  if (!empty($cache[$cid])) {
    return $cache[$cid];
  }

  ctools_include('fields');
  $entities = entity_get_info();
  $relations = relation_get_types();
  $plugins = array();
  $context_types = array();

  foreach ($relations as $relation_name => $relation) {
    $sourcebundles = _relation_ctools_relationships_entity_from_entity_expand_bundles($relation->source_bundles);
    $targetbundles = $relation->directional ? _relation_ctools_relationships_entity_from_entity_expand_bundles($relation->target_bundles) : $sourcebundles;

    foreach ($sourcebundles as $sourceentity_name => $entity_sourcebundles) {
      $sourceentity = $entities[$sourceentity_name];

      foreach ($targetbundles as $targetentity_name => $entity_targetbundles) {
        $targetentity = $entities[$targetentity_name];

        foreach ($entity_sourcebundles as $sourcebundle_name) {
          foreach($entity_targetbundles as $targetbundle_name) {
            if (!isset($sourceentity['bundles'][$sourcebundle_name]) ||
                !isset($sourceentity['bundles'][$targetbundle_name])) {
              continue;
            }

            $name = $relation_name . '-' . $sourceentity_name . ':' . $sourcebundle_name . '-' . $targetentity_name . ':' . $targetbundle_name;
            $plugin_id = $parent . ':' . $name;

            $sourcebundle_title = $sourceentity['bundles'][$sourcebundle_name]['label'];
            $targetbundle_title = $targetentity['bundles'][$targetbundle_name]['label'];

            $plugin = $parent_plugin;
            $replacements = array(
              '@source_entity' => $sourceentity['label'],
              '@target_entity' => $targetentity['label'],
              '@source_bundle' => $sourcebundle_title,
              '@target_bundle' => $targetbundle_title,
              '@relation' => $relation->label,
            );
            $plugin['title'] = t('From @source_bundle (@source_entity) to @target_bundle (@target_entity) via relation @relation', $replacements);
            $plugin['keyword'] = $relation_name;
            $plugin['context name'] = $name;
            $plugin['name'] = $plugin_id;
            $plugin['description'] = t('Creates a @target_bundle context from @source_bundle using the @relation relation.', $replacements);
            $plugin['from entity'] = $sourceentity_name;
            $plugin['from bundle'] = $sourcebundle_name;
            $plugin['to entity'] = $targetentity_name;
            $plugin['to bundle'] = $targetbundle_name;
            $plugin['relation'] = $relation_name;
            $plugin['parent'] = $parent;
            $plugin['required context'] = new ctools_context_required($sourceentity['label'], $sourceentity, array('type' => array($sourcebundle_name)));

            $plugin_entities = array(
              'from' => array($sourceentity_name => $sourceentity),
              'to' => array($targetentity_name => $targetentity),
            );
            drupal_alter('ctools_entity_context', $plugin, $plugin_entities, $plugin_id);

            $plugins[$plugin_id] = $plugin;

            // Add the reverse relation in case of a directional relationship
            if ($relation->directional) {
              $name = $relation_name . '-reverse-' . $targetentity_name . ':' . $targetbundle_name . '-' . $sourceentity_name . ':' . $sourcebundle_name;
              $plugin_id = $parent . ':' . $name;

              $sourcebundle_title = $sourceentity['bundles'][$sourcebundle_name]['label'];
              $targetbundle_title = $targetentity['bundles'][$targetbundle_name]['label'];

              $plugin = $parent_plugin;
              $replacements = array(
                '@source_entity' => $targetentity['label'],
                '@target_entity' => $sourceentity['label'],
                '@source_bundle' => $targetbundle_title,
                '@target_bundle' => $sourcebundle_title,
                '@relation' => $relation->reverse_label,
              );
              $plugin['title'] = t('From @source_bundle (@source_entity) to @target_bundle (@target_entity) via relation @relation', $replacements);
              $plugin['keyword'] = $relation_name;
              $plugin['context name'] = $name . '-reverse';
              $plugin['name'] = $plugin_id;
              $plugin['description'] = t('Creates a @source_bundle context from @target_bundle using the @relation relation.', $replacements);
              $plugin['from entity'] = $targetentity_name;
              $plugin['from bundle'] = $targetbundle_name;
              $plugin['to entity'] = $sourceentity_name;
              $plugin['to bundle'] = $sourcebundle_name;
              $plugin['relation'] = $relation_name;
              $plugin['parent'] = $parent;
              $plugin['required context'] = new ctools_context_required($targetentity['label'], $targetentity, array('type' => array($targetbundle_name)));

              $plugin_entities = array(
                'from' => array($targetentity_name => $targetentity),
                'to' => array($sourceentity_name => $sourceentity),
              );
              drupal_alter('ctools_entity_context', $plugin, $plugin_entities, $plugin_id);

              $plugins[$plugin_id] = $plugin;
            }
          }
        }
      }
    }
  }

  drupal_alter('ctools_entity_contexts', $plugins);

  $cache[$cid] = $plugins;
  return $plugins;
}

/**
 * Return a new context based on an existing context.
 */
function relation_ctools_relationships_entity_from_entity_context($context, $conf) {
  $delta = !empty($conf['delta']) ? intval($conf['delta']) : 0;
  $plugin = $conf['name'];

  list($plugin, $plugin_name) = explode(':', $plugin, 2);

  list($relation_name, $from_entity_bundle, $to_entity_bundle) = explode('-', $plugin_name, 3);
  list($from_entity, $from_bundle) = explode(':', $from_entity_bundle, 2);
  list($to_entity, $to_bundle) = explode(':', $to_entity_bundle, 2);

  $entity_info = entity_get_info($from_entity);
  if (empty($context->data) || !isset($context->data->{$entity_info['entity keys']['id']})) {
    // Unfortunately, we can't filter on bundle here.
    return ctools_context_create_empty('entity:' . $to_entity, NULL);
  }

  if (isset($context->data->{$entity_info['entity keys']['id']})) {
    list($id, $vid, $bundle) = entity_extract_ids($from_entity, $context->data);

    if ($bundle === $from_bundle) {
      $query = relation_query($from_entity, $id);
      $query->entityCondition('bundle', $relation_name);
      $query->range(0, 1);
      $results = $query->execute();
      $result = reset($results); // Fetch the first relation instance
      if ($result) {
        $relation = relation_load($result->rid);
        $entities = field_get_items('relation', $relation, 'endpoints');

        foreach ($entities as $entity) {
          // If it is the same as the source entity, skip
          if ($entity['entity_type'] == $from_entity && $entity['entity_id'] == $id) {
            continue;
          }
          // If the entity doesn't have the right entity type, skip
          if ($entity['entity_type'] != $to_entity) {
            continue;
          }
          // Load the entity
          $entity = entity_load($to_entity, array($entity['entity_id']));
          $entity = reset($entity);
          if (!$entity) {
            continue;
          }
          list($id, $vid, $bundle) = entity_extract_ids($to_entity, $entity);
          // If the target entity doesn't have the right bundle, skip
          if ($bundle != $to_bundle) {
            continue;
          }
          return ctools_context_create('entity:' . $to_entity, $entity);
        }
      }
    }
  }

  return ctools_context_create_empty('entity:' . $to_entity, NULL);
}

/**
 * Expands an bundle array.
 *
 * Input:
 * array(
 *   'entitytype1:bundle1',
 *   'entitytype1:bundle2',
 *   'entitytype2:*',
 * );
 *
 * Output:
 * array(
 *   'entitytype1' => array(
 *     'bundle1',
 *     'bundle2',
 *   ),
 *   'entitytype2' => array(
 *     'bundle1',
 *     'bundle2',
 *     'bundle3',
 *     'bundle4',
 *   ),
 */
function _relation_ctools_relationships_entity_from_entity_expand_bundles(array $bundlekeys) {
  $bundles = array();

  foreach ($bundlekeys as $bundlekey) {
    list($entity_type, $bundle) = explode(':', $bundlekey, 2);

    if ($bundle === '*') {
      $info = entity_get_info($entity_type);
      $bundles[$entity_type] = array_keys($info['bundles']);
    }
    else {
      if (!isset($bundles[$entity_type])) {
        $bundles[$entity_type] = array();
      }
      $bundles[$entity_type][] = $bundle;
    }
  }

  return $bundles;
}
